package tools;

import jason.JasonException;
import jason.asSemantics.DefaultArithFunction;
import jason.asSemantics.TransitionSystem;
import jason.asSyntax.NumberTerm;
import jason.asSyntax.Term;
import jason.asSyntax.ASSyntax;

/** 
  <p>Function: <b><code>tools.random(Action,N)</code></b>: wraps java.util.Random class and allows to generate different pseudorandom numbers.</p>
  
  <p>Parameter:<ul>
  
  <li>- action (atom): name of the fuction to be invoked (i.e. <code>setSeed</code>, <code>nextDouble</code>, <code>nextInt</code>, <code>nextBoolean</code> or <code>nextGaussian</code>)<br/>
  <li>- value (number): if action is <code>setSeed</code>, the new seed for the generator<br/>
  
  </ul>
  
  <p>Example:<ul> 

  <li><code>.random2(setSeed, 1)</code>: sets 1 as the seed of the pseudorandom number generator and returns 1 (i.e. the new seed).</li>
  <li><code>.random2(nextDouble)</code>: returns one random number between 0 and 1.</li>
  <li><code>.random2(nextInt)</code>: returns one random integer number, uniformly distributed  amongst all 2^32 possible int values.</li>
  <li><code>.random2(nextBoolean)</code>: returns one random, uniformly distributed, boolean value (i.e. <code>true</code> or <code>false</code>).</li>
  <li><code>.random2(nextGaussian)</code>: returns one Gaussian ("normally") distributed double value with mean 0.0 and standard deviation 1.0.</li>

  </ul>

@author Francisco Grimaldo 

@see jason.functions.Random

*/
public class random extends DefaultArithFunction  {

	private static final long serialVersionUID = 5773379123465602111L;

	private static java.util.Random random = null; 
	
    static {
		random = new java.util.Random();
    }

	static	public void setSeed(long s)
	{
		random.setSeed(s);
	}
	
	@Override
    public String getName() {
        return "tools.random";
    }

    protected void checkArguments(Term[] args) throws JasonException {
        if (!args[0].isAtom())
            throw new JasonException("first argument must be an atom.");                
        if (args[0].equals(ASSyntax.createAtom("setSeed")) && (args.length == 1 || !args[1].isNumeric()) )
            throw new JasonException(" setSeed needs a numeric second argument.");                
        else if (args[0].equals(ASSyntax.createAtom("nextDouble")) ||
			 args[0].equals(ASSyntax.createAtom("nextInt")) ||
		     args[0].equals(ASSyntax.createAtom("nextBoolean")) ||
			 args[0].equals(ASSyntax.createAtom("nextGaussian"))  	)
			if (args.length == 2)
				throw new JasonException(args[0] + " does not need a second argument.");
    }
    
    @Override
    public double evaluate(TransitionSystem ts, Term[] args) throws Exception {		
        checkArguments(args);

		if (args[0].equals(ASSyntax.createAtom("setSeed"))) {
			long seed = (long) ((NumberTerm)args[1]).solve(); 
			setSeed(seed);
			return (double) seed;
		} 
		else if  (args[0].equals(ASSyntax.createAtom("nextDouble"))) 
		{ 
			return random.nextDouble();
		}
		else if  (args[0].equals(ASSyntax.createAtom("nextInt"))) 
		{ 
			return random.nextInt();
		}
		else if  (args[0].equals(ASSyntax.createAtom("nextBoolean"))) 
		{ 
			return (random.nextBoolean() == true) ? 1 : 0;
		}
		else if  (args[0].equals(ASSyntax.createAtom("nextGaussian"))) 
		{ 
			return random.nextGaussian();
		}
		else
			throw new JasonException(args[0] + " is not a valid first argument.");

    }

    @Override
    public boolean checkArity(int a) {
        return (a == 1 || a == 2);
    }
    
}
